#include <sourcemod>
#include <sdktools>
#include <hgr>

#pragma semicolon 1

#define PLUGIN_VERSION "1.0"

public Plugin:myinfo = 
{
	name = "Hook Grab Rope Limit Use",
	author = "Sheepdude",
	description = "Sets map and round limits for hook, grab, and rope use",
	version = PLUGIN_VERSION,
	url = "http://www.clan-psycho.com"
};

// Convar handles
new Handle:h_cvarHookMap;
new Handle:h_cvarHookRound;
new Handle:h_cvarGrabMap;
new Handle:h_cvarGrabRound;
new Handle:h_cvarRopeMap;
new Handle:h_cvarRopeRound;

// Convar variables
new g_cvarHookMap;
new g_cvarHookRound;
new g_cvarGrabMap;
new g_cvarGrabRound;
new g_cvarRopeMap;
new g_cvarRopeRound;

// Plugin variables
new bool:g_HookedRound;
new g_MapUses[MAXPLAYERS+1][3];
new g_RoundUses[MAXPLAYERS+1][3];

enum HGRAction
{
	Hook = 0, /** User is using hook */
	Grab = 1, /** User is using grab */
	Rope = 2, /** User is using rope */
};

public OnPluginStart()
{
	// Event hooks
	g_HookedRound = HookEventEx("round_start", OnRoundStart);
	
	// Public convar
	CreateConVar("sm_hgrlimit_version", PLUGIN_VERSION, "[HGR] Plugin version", FCVAR_DONTRECORD|FCVAR_NOTIFY|FCVAR_PLUGIN|FCVAR_REPLICATED|FCVAR_SPONLY);
	
	// Map Convars
	h_cvarHookMap = CreateConVar("sm_hgrlimit_hook_map", "0", "Map limit for hook uses", FCVAR_NOTIFY);
	h_cvarGrabMap = CreateConVar("sm_hgrlimit_grab_map", "0", "Map limit for grab uses", FCVAR_NOTIFY);
	h_cvarRopeMap = CreateConVar("sm_hgrlimit_rope_map", "0", "Map limit for rope uses", FCVAR_NOTIFY);
	HookConVarChange(h_cvarHookMap, ConvarChanged);
	HookConVarChange(h_cvarGrabMap, ConvarChanged);
	HookConVarChange(h_cvarRopeMap, ConvarChanged);
	
	// Round convars
	if(g_HookedRound)
	{
		h_cvarHookRound = CreateConVar("sm_hgrlimit_hook_round", "0", "Round limit for hook uses", FCVAR_NOTIFY);
		h_cvarGrabRound = CreateConVar("sm_hgrlimit_grab_round", "0", "Round limit for grab uses", FCVAR_NOTIFY);
		h_cvarRopeRound = CreateConVar("sm_hgrlimit_rope_round", "0", "Round limit for rope uses", FCVAR_NOTIFY);
		HookConVarChange(h_cvarHookRound, ConvarChanged);
		HookConVarChange(h_cvarGrabRound, ConvarChanged);
		HookConVarChange(h_cvarRopeRound, ConvarChanged);
	}
	
	// Auto-generate configuration file
	AutoExecConfig(true, "hgrlimit");
}

/**********
 *Forwards*
***********/

public OnConfigsLoaded()
{
	UpdateAllConvars();
	for(new i = 0; i <= MaxClients; i++)
	{
		for(new HGRAction:k = Hook; k <= Rope; k++)
		{
			g_MapUses[i][k] = 0;
			g_RoundUses[i][k] = 0;
		}
	}
}

/********
 *Events*
*********/

public OnRoundStart(Handle:event, const String:name[], bool:dontBroadcast)
{
	for(new i = 0; i <= MaxClients; i++)
	{
		for(new HGRAction:k = Hook; k <= Rope; k++)
			g_RoundUses[i][k] = 0;
	}
}

public Action:HGR_OnClientHook(client)
{
	if(g_cvarHookMap > 0 && g_MapUses[client][Hook] > g_cvarHookMap)
		return Plugin_Handled;
	if(g_HookedRound && g_cvarHookRound > 0)
	{
		if(g_RoundUses[client][Hook] > g_cvarHookRound)
			return Plugin_Handled;
		g_RoundUses[client][Hook]++;
	}
	g_MapUses[client][Hook]++;
	return Plugin_Continue;
}

public Action:HGR_OnClientGrab(client)
{
	if(g_cvarGrabMap > 0 && g_MapUses[client][Grab] > g_cvarGrabMap)
		return Plugin_Handled;
	if(g_HookedRound && g_cvarGrabRound > 0)
	{
		if(g_RoundUses[client][Grab] > g_cvarGrabRound)
			return Plugin_Handled;
		g_RoundUses[client][Grab]++;
	}
	g_MapUses[client][Grab]++;
	return Plugin_Continue;
}

public Action:HGR_OnClientRope(client)
{
	if(g_cvarRopeMap > 0 && g_MapUses[client][Rope] > g_cvarRopeMap)
		return Plugin_Handled;
	if(g_HookedRound && g_cvarRopeRound > 0)
	{
		if(g_RoundUses[client][Rope] > g_cvarRopeRound)
			return Plugin_Handled;
		g_RoundUses[client][Rope]++;
	}
	g_MapUses[client][Rope]++;
	return Plugin_Continue;
}

/*********
 *Convars*
**********/

public ConvarChanged(Handle:cvar, const String:oldVal[], const String:newVal[])
{
	if(cvar == h_cvarHookMap)
		g_cvarHookMap = GetConVarInt(h_cvarHookMap);
	else if(cvar == h_cvarGrabMap)
		g_cvarGrabMap = GetConVarInt(h_cvarGrabMap);
	else if(cvar == h_cvarRopeMap)
		g_cvarRopeMap = GetConVarInt(h_cvarRopeMap);
	else if(cvar == h_cvarHookRound)
		g_cvarHookRound = GetConVarInt(h_cvarHookRound);
	else if(cvar == h_cvarGrabRound)
		g_cvarGrabRound = GetConVarInt(h_cvarGrabRound);
	else if(cvar == h_cvarRopeRound)
		g_cvarRopeRound = GetConVarInt(h_cvarRopeRound);
}

UpdateAllConvars()
{
	g_cvarHookMap = GetConVarInt(h_cvarHookMap);
	g_cvarGrabMap = GetConVarInt(h_cvarGrabMap);
	g_cvarRopeMap = GetConVarInt(h_cvarRopeMap);
	if(g_HookedRound)
	{
		g_cvarHookRound = GetConVarInt(h_cvarHookRound);
		g_cvarGrabRound = GetConVarInt(h_cvarGrabRound);
		g_cvarRopeRound = GetConVarInt(h_cvarRopeRound);
	}
}